'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var Babel = require('@babel/standalone');
var merge = _interopDefault(require('deepmerge'));
var memoize = _interopDefault(require('fast-memoize'));
var React = require('react');
var React__default = _interopDefault(React);
var equal = _interopDefault(require('fast-deep-equal'));
var PropTypes = _interopDefault(require('prop-types'));
var reactDom = require('react-dom');
var server = require('react-dom/server');

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

function _inherits(subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function");
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      writable: true,
      configurable: true
    }
  });
  if (superClass) _setPrototypeOf(subClass, superClass);
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};

  var target = _objectWithoutPropertiesLoose(source, excluded);

  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

function _possibleConstructorReturn(self, call) {
  if (call && (typeof call === "object" || typeof call === "function")) {
    return call;
  }

  return _assertThisInitialized(self);
}

var createIIFEProgram = function createIIFEProgram(t, body) {
  return t.program([t.expressionStatement(t.callExpression(t.functionExpression(null, [], t.blockStatement(body)), []))]);
};

var exportToIIFEPlugin = function exportToIIFEPlugin(_ref) {
  var t = _ref.types;
  return {
    visitor: {
      ExportDefaultDeclaration: function ExportDefaultDeclaration(path) {
        var program = path.find(function (parentPath) {
          return parentPath.isProgram();
        });
        var declaration = path.node.declaration;

        if (t.isClassDeclaration(declaration)) {
          path.replaceWith(declaration);
          program.pushContainer("body", t.ReturnStatement(t.identifier(declaration.id.name)));
          return;
        }

        program.pushContainer("body", t.ReturnStatement(t.identifier(declaration.name)));
        path.remove();
      },
      Program: {
        exit: function exit(path) {
          path.replaceWith(createIIFEProgram(t, path.node.body));
          path.stop();
        }
      }
    }
  };
};

var resolverId = "__resolverFn__";

var createVariableDeclaration = function createVariableDeclaration(t, specifier, source) {
  return t.variableDeclaration("const", [t.variableDeclarator(t.identifier(specifier.local.name), t.callExpression(t.identifier(resolverId), [t.stringLiteral(source.value)]))]);
};

var createDestructedDeclaration = function createDestructedDeclaration(t, specifier, source) {
  return t.variableDeclaration("const", [t.variableDeclarator(t.objectPattern([t.objectProperty(t.identifier(specifier.imported.name), t.identifier(specifier.local.name), false, true)]), t.callExpression(t.identifier(resolverId), [t.stringLiteral(source.value)]))]);
};

var importResolverPlugin = function importResolverPlugin(_ref) {
  var t = _ref.types;
  return {
    visitor: {
      ImportDeclaration: function ImportDeclaration(path) {
        var replacements = path.node.specifiers.map(function (specifier) {
          if (t.isImportDefaultSpecifier(specifier) || t.isImportNamespaceSpecifier(specifier)) {
            return createVariableDeclaration(t, specifier, path.node.source);
          }

          if (t.isImportSpecifier(specifier)) {
            return createDestructedDeclaration(t, specifier, path.node.source);
          }

          return null;
        });
        path.replaceWithMultiple(replacements);
      }
    }
  };
};

/**
 * Evaluates passed source code, uses passed resolver to resolve imports.
 *
 * @param {String} source A string that contains the source code
 * @param {Function} resolver A function for the imports resolution
 * @param {Object} resolverContext A context of resolver, will be passed as second argument to it
 */

var evalSource = function evalSource(source, resolver, resolverContext) {
  var resolverWithContext = function resolverWithContext(importName) {
    return resolver(importName, resolverContext);
  }; // eslint-disable-next-line no-new-func


  var evalWithResolver = new Function(resolverId, "return ".concat(source));
  return evalWithResolver(resolverWithContext);
};

Babel.registerPlugin("export-to-iife", exportToIIFEPlugin);
Babel.registerPlugin("import-resolver", importResolverPlugin);
/**
 * Transform passed ES6 code to the evaluable code using plugins and a passed config.
 *
 * @param {Object} babelConfig A config for Babel
 * @param {String} source A string that contains the source code
 *
 * @return {String}
 */

var transformSource = function transformSource(babelConfig, source) {
  var config = merge({
    plugins: ["import-resolver", "export-to-iife"],
    presets: ["react"]
  }, babelConfig);

  var _Babel$transform = Babel.transform(source, config),
      code = _Babel$transform.code;

  return code;
};

var transformSource$1 = memoize(transformSource);

/**
 * Creates a valid React Element from source.
 *
 * @param {Object} babelConfig A config for Babel
 * @param {Function} resolver A function for the imports resolution
 * @param {Object} resolverContext A context of resolver, will be passed as second argument to it
 * @param {String} source A string that contains the source code
 */

var createElementFromSource = function createElementFromSource(babelConfig, resolver, resolverContext, source) {
  var transformed = transformSource$1(babelConfig, source);
  var component = evalSource(transformed, resolver, resolverContext);
  return typeof component === "function" ? React.createElement(component) : component;
};

var noop = function noop() {};

// eslint-disable-next-line

var SourceRender =
/*#__PURE__*/
function (_Component) {
  _inherits(SourceRender, _Component);

  function SourceRender() {
    var _getPrototypeOf2;

    var _this;

    _classCallCheck(this, SourceRender);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _possibleConstructorReturn(this, (_getPrototypeOf2 = _getPrototypeOf(SourceRender)).call.apply(_getPrototypeOf2, [this].concat(args)));

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "state", {
      element: null
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "handleRef", function (c) {
      _this.ref = c;
    });

    return _this;
  }

  _createClass(SourceRender, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      this.renderElement();
    }
  }, {
    key: "shouldComponentUpdate",
    value: function shouldComponentUpdate(nextProps) {
      return !equal(this.props, nextProps);
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate() {
      this.renderElement();
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      if (this.frameId) cancelAnimationFrame(this.frameId);
      if (this.ref) reactDom.unmountComponentAtNode(this.ref);
    }
  }, {
    key: "renderElement",
    value: function renderElement() {
      var _this2 = this;

      this.frameId = requestAnimationFrame(function () {
        var _this2$props = _this2.props,
            babelConfig = _this2$props.babelConfig,
            onError = _this2$props.onError,
            onSuccess = _this2$props.onSuccess,
            resolver = _this2$props.resolver,
            source = _this2$props.source,
            rest = _objectWithoutProperties(_this2$props, ["babelConfig", "onError", "onSuccess", "resolver", "source"]);

        var prevElement = _this2.state.element;
        reactDom.unmountComponentAtNode(_this2.ref);

        try {
          var element = createElementFromSource(babelConfig, resolver, rest, source);
          var markup = server.renderToStaticMarkup(element);
          reactDom.render(element, _this2.ref);
          onSuccess(null, _objectSpread({}, _this2.props, {
            element: element,
            markup: markup
          }));

          _this2.setState({
            element: element
          });
        } catch (error) {
          onError(error, _this2.props);
          reactDom.render(prevElement, _this2.ref);
        }
      });
    }
  }, {
    key: "render",
    value: function render() {
      return React__default.createElement("div", {
        ref: this.handleRef
      });
    }
  }]);

  return SourceRender;
}(React.Component);

_defineProperty(SourceRender, "propTypes", {
  /** A config for Babel. */
  babelConfig: PropTypes.object,

  /**
   * A callback that will be called after the render process that ended with an error.
   *
   * @param {Error} error
   * @param {Object} props
   */
  onError: PropTypes.func,

  /**
   * A callback that will be called after the render process that ended successfully.
   *
   * @param {Error} error
   * @param {Object} props
   */
  onSuccess: PropTypes.func,

  /**
   * A function for the imports resolution.
   *
   * @param {String} importPath
   */
  resolver: PropTypes.func.isRequired,

  /** A string that contains the source code. */
  source: PropTypes.string.isRequired
});

_defineProperty(SourceRender, "defaultProps", {
  babelConfig: {},
  onError: noop,
  onSuccess: noop
});

module.exports = SourceRender;
